/*
** hist v1.2
**
** Character Histogram
**
** Copyright (C) 19.11.1990 by Andreas Ley <andreas.ley@kit.edu>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
** This program has been tested on a HP9000/715 with HP-UX A.09.07
** In this environment, neither lint -u nor gcc -Wall -pedantic -fsyntax-only
** produce any messages. If you encounter any errors or need to make any
** changes to port it to another platform, please contact me.
**
** Version history
**
** Version 1.2 - 17.2.1998
**	Cleanup
**
** Version 1.1 - 3.8.1991
**	Verbose mode added
**
** Version 1.0 - 19.11.1990
**	Initial version
*/

static char copyright[] = "@(#)Copyright (C) 1990-1998 by Andreas Ley (andreas.ley@kit.edu)";
static char sccsid[] = "@(#)hist v1.2 - Character Histogram";


#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifndef FALSE
#define	FALSE	(0)		/* This is the naked Truth */
#define	TRUE	(1)		/* and this is the Light */
#endif


char	*image;
int	debug=0,verbose=FALSE,unused=FALSE;

unsigned int	hist[256];
char		*info[256]={
		"NUL","SOH","STX","ETX","EOT","ENQ","ACK","BEL",
		"BS","HT","NL","VT","NP","CR","SO","SI",
		"DLE","DC1","DC2","DC3","DC4","NAK","SYN","ETB",
		"CAN","EM","SUB","ESC","FS","GS","RS","US",
		"SP" };
enum	{OCT,DEC,HEX} mode=DEC;
char	*fmtnum[]={ "\\%03o", "%3d", "0x%02x" };
char	*fmtrun[]={ "%03o", "%d", "%02x" };


static void makehist(src)
FILE	*src;
{
	unsigned char	c;

	if (debug)
		(void)fprintf(stderr,"[%d] makehist(0x%08lx)\n",__LINE__,(unsigned long)src);

	while (c=(unsigned char)getc(src),!feof(src)&&!ferror(src))
		hist[c]++;
}



static void showhist()
{
	int	i;
	char	tmp[2],format[160];

	if (debug)
		(void)fprintf(stderr,"[%d] showhist()\n",__LINE__);

	tmp[1]='\0';
	if (unused) {
		(void)sprintf(format,verbose?"%s\t%%s\n":" %s",verbose?fmtnum[mode]:fmtrun[mode]);
		if (debug)
			(void)fprintf(stderr,"[%d] showhist: format=\"%s\"\n",__LINE__,format);
		for (i=0;i<256;i++)
			if (hist[i]==0)
				(void)printf(format,i,info[i]?info[i]:(tmp[0]=i,tmp));
		(void)printf(verbose?"":"\n");
	}
	else {
		(void)sprintf(format,verbose?"%s  %%5d\t%%s\n":"%s\n",verbose?fmtnum[mode]:fmtrun[mode]);
		if (debug)
			(void)fprintf(stderr,"[%d] showhist: format=\"%s\"\n",__LINE__,format);
		for (i=0;i<256;i++)
			if (hist[i])
				(void)printf(format,i,hist[i],info[i]?info[i]:(tmp[0]=i,tmp));
	}
}



static void usage()
{
	(void)fprintf(stderr,"Usage: %s [-h] [-v] [-u] [-x|-o|-b] [filename...]\n",image);
	(void)fprintf(stderr,"-v  verbose output\n");
	(void)fprintf(stderr,"-u  show unused characters\n");
	(void)fprintf(stderr,"-x  hex output\n");
	(void)fprintf(stderr,"-o  octal output\n");
	exit(1);
}


int main(argc,argv)
int	argc;
char	*argv[];
{
	int		c;
	extern int	optind;
	FILE		*src;

	if ((image=strrchr(argv[0],'/')))
		image++;
	else
		image=argv[0];

	while ((c=getopt(argc,argv,"Dodxuvh?"))!=EOF)
		switch ((char)c) {
		case 'D':
			debug++;
			break;
		case 'o':
			mode=OCT;
			break;
		case 'd':
			mode=DEC;
			break;
		case 'x':
			mode=HEX;
			break;
		case 'u':
			unused=TRUE;
			break;
		case 'v':
			verbose=TRUE;
			break;
		case 'h':
			(void)fprintf(stderr,"%s\n",sccsid+4);
			(void)fprintf(stderr,"%s\n",copyright+4);
		case '?':
			usage();
		}

	for (c=0;c<256;c++) 
		hist[c]=0;

	for (c=33;c<256;c++) 
		info[c]=NULL;
	info[127]="DEL";

	if (optind<argc) {
		for (;optind<argc;optind++)
			if (strcmp(argv[optind],"-")) {
				if ((src=fopen(argv[optind],"r"))) {
					makehist(src);
					(void)fclose(src);
				}
				else {
					(void)fprintf(stderr,"%s: cannot open ",image);
					perror(argv[optind]);
				}
			}
			else
				makehist(stdin);
	}
	else
		makehist(stdin);

	showhist();
	return(0);
}
